"""
Based on annotation_db/microdraw.py
"""

try:
    import MySQLdb as MySQLdb
except ImportError:
    import pymysql as MySQLdb

import json
import sys

assert sys.version_info[0] == 3, "WARNING: Python 3 is required to handle umlaute correctly."


class MicrodrawDb(object):
    def __init__(self, host):
        self.host = host
        self.db = MySQLdb.connect(host=self.host,
                                  user="microdraw",
                                  passwd="microdraw",
                                  db="Interact")
        self.cursor = self.db.cursor()

    def export_annotations(self, json_path, user_name, sections=None):
        available_sections = self.get_available_slices(json_path=json_path,
                                                       user_name=user_name)
        if sections is None:
            sections = available_sections
        else:
            available_sections = set(available_sections)
            sections = [s for s in sections if s in available_sections]

        annotation_per_section = {}
        # Query data for each section individually
        for section in sections:
            origin = json.dumps({"appName": "microdraw",
                                 "source": json_path,
                                 # Important: Slice number must be string!
                                 "slice": f"{section:04d}",
                                 "user": user_name}, sort_keys=True, separators=(",", ":"))

            annotation_per_section[int(section)] = self.get_latest_regions(origin)
        return annotation_per_section

    def import_annotations(self, json_path, user_name, annotations, overwrite=False):
        import random
        from datetime import datetime
        from atlaslib.annotations import serialize_structures

        section_str = f"{annotations.section:04d}"

        timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        origin = {"appName": "microdraw",
                  "source": json_path,
                  "slice": section_str,
                  "user": user_name}
        origin = json.dumps(origin,
                            sort_keys=True,
                            separators=(",", ":"),
                            ensure_ascii=False)
        # Serialize annotations to microdraw compatible format
        annotation_data = serialize_structures(structures=annotations.structures,
                                               colors=annotations.colors)
        # Annotation data has to provide a hash, otherwise saving in microdraw does not work
        dummy_hash = f"dummy_hash_{random.randint(1, 999999999):09d}"
        annotation_data["Hash"] = dummy_hash

        sql_query = None
        if not overwrite:
            # Check if annotations already exist for this key
            unique_id, existing_annotations = self.get_latest_annotations(origin, return_id=True)
            if existing_annotations:
                # Update the existing annotations with the new annotations
                def _update_field(field_name):
                    if field_name in existing_annotations and field_name in annotation_data:
                        existing_annotations[field_name].extend(annotation_data[field_name])
                    elif field_name in annotation_data:
                        existing_annotations[field_name] = annotation_data[field_name]

                _update_field("Regions")
                _update_field("Points")
                _update_field("Text")

                # Update hash, so saving works properly
                existing_annotations["Hash"] = dummy_hash

                # Prepare update SQL statement
                existing_annotations = json.dumps(existing_annotations,
                                                  sort_keys=True,
                                                  separators=(",", ":"),
                                                  ensure_ascii=False)
                sql_query = f"UPDATE KeyValue SET myValue = '{existing_annotations}' WHERE UniqueID = {unique_id}"
                # Prevent insertion
                insert_new = False
            else:
                # No existing annotations, insert new data
                insert_new = True
        else:
            # Just insert as new data, ignoring existing data
            insert_new = True

        if insert_new:
            # Just insert into the database, overwriting existing annotations
            value = json.dumps(annotation_data, sort_keys=True, separators=(",", ":"), ensure_ascii=False)
            sql_query = f"INSERT INTO KeyValue (myTimestamp, myOrigin, myKey, myValue) values ('{timestamp}', '{origin}', 'regionPaths', '{value}')"

        # Perform insert operation
        self.cursor.execute(sql_query)
        self.db.commit()

    def get_available_slices(self, json_path, user_name):
        # Wildcard to find all annotations for given JSON path and username
        origin = "%" + json_path + "%" + user_name + "%"
        sql_query = f"SELECT DISTINCT myOrigin FROM Interact.KeyValue WHERE myOrigin like '{origin}' AND myKey = 'regionPaths'"
        self.cursor.execute(sql_query)
        res = self.cursor.fetchall()
        sources = []

        for r in res:
            parsed = json.loads(r[0])
            # is this in the new format?
            if "slice" in parsed:
                sources.append([parsed["source"], parsed["slice"]])
            else:
                sources.append([parsed["source"].split("@")[0], parsed["source"].split("@")[1]])
        # We only need the slice number
        slice_numbers = [int(s[1]) for s in sources]
        return sorted(slice_numbers)

    def get_latest_annotations(self, origin, return_id=False):
        """
        Gets the latest annotations for user and image specified by origin.
        """
        sql_query = f"SELECT myValue, UniqueID FROM Interact.KeyValue WHERE myOrigin = '{origin}' AND myKey = 'regionPaths' ORDER BY myTimestamp DESC LIMIT 1"
        self.cursor.execute(sql_query)
        res = self.cursor.fetchone()

        if res:
            if return_id:
                # Return ID and value
                return res[1], json.loads(res[0])
            else:
                return json.loads(res[0])
        else:
            if return_id:
                return None, None
            else:
                return None

    # noinspection PyTypeChecker
    def get_latest_regions(self, origin):
        """
        Get most recent annotations for user and image specified by origin.
        """
        regions = self.get_latest_annotations(origin)

        if not regions:
            return None

        # Convert all segment of paths into proper python lists
        for i in range(len(regions["Regions"])):
            if "segments" in regions["Regions"][i]["path"][1]:
                regions["Regions"][i]["path"][1]["segments"] = path_dict_to_list(regions["Regions"][i]["path"][1]["segments"])
        return regions

    def commit(self):
        self.db.commit()


def path_dict_to_list(path):
    """
    Converts given points which are represented as dictionary
    into default python lists.
    """
    res_path = []
    for pt in path:
        if isinstance(pt, dict):
            res_path.append([pt["x"], pt["y"]])
        else:
            res_pt = []
            for ele in pt:
                if isinstance(ele, dict):
                    res_pt.append([ele["x"], ele["y"]])
                else:
                    res_pt.append(ele)
            res_path.append(res_pt)
    return res_path
