#!/usr/bin/env python3
"""
Model classes for ATLaS UI
"""
from marshmallow import Schema, fields, post_load


class JobConfigurationSchema(Schema):
    account = fields.String()
    time = fields.String()
    job_type = fields.String()
    gres = fields.String(allow_none=True)
    mail_type = fields.String(allow_none=True)
    mail_user = fields.String(allow_none=True)
    num_nodes = fields.Integer(allow_none=True)
    partition = fields.String(allow_none=True)
    reservation = fields.String(allow_none=True)
    kwargs = fields.String(allow_none=True)


    # noinspection PyUnusedLocal
    @post_load
    def create_job_configuration(self, data, **kwargs):
        return JobConfiguration(**data)


class JobSchema(Schema):
    job_id = fields.Integer()
    status = fields.String(allow_none=True)
    job_configuration = fields.Nested(JobConfigurationSchema)

    # noinspection PyUnusedLocal
    @post_load
    def create_job(self, data, **kwargs):
        return Job(**data)


class AnnotationSchema(Schema):
    section = fields.Integer()
    name = fields.String()
    original_name = fields.String(allow_none=True)
    path = fields.String()

    # noinspection PyUnusedLocal
    @post_load
    def create_annotation(self, data, **kwargs):
        return Annotation(**data)


class TaskSchema(Schema):
    task_id = fields.Integer()
    name = fields.String()
    created = fields.Integer()
    modified = fields.Integer()
    annotations = fields.List(fields.Nested(AnnotationSchema))
    jobs = fields.List(fields.Nested(JobSchema))
    no_aux_labels = fields.Boolean(allow_none=True)

    # noinspection PyUnusedLocal
    @post_load
    def create_task(self, data, **kwargs):
        return Task(**data)


class ProjectSchema(Schema):
    project_id = fields.Integer()
    name = fields.String()
    brain = fields.Integer()
    owner = fields.String()
    tasks = fields.List(fields.Nested(TaskSchema))
    created = fields.Integer()
    modified = fields.Integer()

    # noinspection PyUnusedLocalname
    @post_load
    def create_project(self, data, **kwargs):
        return Project(**data)


class Project(object):
    def __init__(self,
                 project_id=None,
                 name=None,
                 brain=None,
                 owner=None,
                 tasks=None,
                 jobs=None,
                 created=None,
                 modified=None):
        self.project_id = project_id
        self.name = name
        self.brain = brain
        self.owner = owner
        self.tasks = tasks
        self.jobs = jobs

        if self.name is None:
            self.name = "Untitled project"

        if self.tasks is None:
            self.tasks = []

        if self.jobs is None:
            self.jobs = []

        self.created = created
        self.modified = modified

    def __repr__(self):
        return f"Project(project_id={self.project_id}, name={self.name}, owner={self.owner})"


class Task(object):
    def __init__(self,
                 task_id=None,
                 name=None,
                 created=None,
                 modified=None,
                 annotations=None,
                 jobs=None,
                 no_aux_labels=None):
        self.task_id = task_id
        self.name = name
        self.annotations = annotations
        self.jobs = jobs
        self.no_aux_labels = no_aux_labels

        if self.name is None:
            self.name = "Untitled task"

        if self.annotations is None:
            self.annotations = []

        if self.jobs is None:
            self.jobs = []

        if self.no_aux_labels is None:
            self.no_aux_labels = False

        self.created = created
        self.modified = modified

    def __repr__(self):
        return f"Task(task_id={self.task_id}, name={self.name})"


class Annotation(object):
    def __init__(self,
                 section=None,
                 original_name=None,
                 name=None,
                 path=None):
        self.section = section
        self.name = name
        self.path = path

    def __repr__(self):
        return f"Annotation(section={self.section}, name={self.name})"


class Job(object):
    def __init__(self,
                 job_id=None,
                 status=None,
                 job_configuration=None):
        self.job_id = job_id
        self.status = status
        self.job_configuration = job_configuration

    def __repr__(self):
        return f"Job(job_id={self.job_id})"


class JobConfiguration(object):
    def __init__(self,
                 account=None,
                 time=None,
                 job_type=None,
                 gres=None,
                 mail_type=None,
                 mail_user=None,
                 num_nodes=None,
                 partition=None,
                 reservation=None,
                 kwargs=None):
        self.account = account
        self.time = time
        self.job_type = job_type
        self.gres = gres
        self.mail_type = mail_type
        self.mail_user = mail_user
        self.num_nodes = num_nodes
        self.partition = partition
        self.reservation = reservation
        self.kwargs = kwargs
